#include "BarinkEngine.h"

extern EngineStatistics ES;
BarinkEngine::Renderer renderer;
const unsigned int MS_PER_UPDATE = 2;
int main(int argc, char* argv[]) {
	// Setup performance sampler 
	EngineInstrumentation::PerfomanceSamplerInit();

	// Startup services 
	BarinkWindow MainWindow =  BarinkWindow(800, 600);

	renderer = BarinkEngine::Renderer();
	InputSystem = BarinkEngine::InputManager();

	ES = EngineStatistics{};
	InputSystem.attach(&MainWindow);

	GUIManager GUISystem = GUIManager(&MainWindow);

	glEnable(GL_DEPTH_TEST);

	// First call to setup game
	{
		PerfSampler("Start");
		Start();
	}


	double previous = glfwGetTime();
	double lag = 0.0;

	// Runtime loop
	while (!MainWindow.WindowShouldClose()) 
	{
	
		double current = glfwGetTime();
		double elapsed = current - previous;
		previous = current;
		lag += elapsed;

		//EngineInstrumentation::Update(); // Todo this does nothing right now and is therefor disabled

		// Execute main logic 
		{
			PerfSampler("PollEvents");
			InputSystem.PollEvents();

		}
		
		{
			PerfSampler("Update");
			while (lag >= MS_PER_UPDATE) {
				Update();
				lag -= MS_PER_UPDATE;
			}
		}

		{
			PerfSampler("Render");
			Render();
		}
		
		{
			PerfSampler("GUI-Render");
			GUISystem.Render();
		}
		

		{
			PerfSampler("BufferSwap");
			MainWindow.SwapBuffers();
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		}
		
	}

	// Shutdown game
	{
		PerfSampler("Stop");
		Stop();
	}
	
	// Shutdown Services

	return 0;
}

